/// TKS_SharedBuffer.cpp
///
/// (c) 2005-2023 Bastian Spiegel <bs@tkscript.de> 
///     - distributed under terms of the GNU general public license (GPL).
///
///

#include <string.h>
#include "tks.h"
#include "PTN_Node.h"
#include "YAC_Buffer.h"
#include "TKS_SharedBuffer.h"

#ifdef YAC_WIN32
#include <windows.h> 
#endif

#include <time.h>


TKS_SharedBuffer::TKS_SharedBuffer(void) : YAC_Buffer() {
   class_ID = TKS_CLID_SHAREDBUFFER;
   shmOwner = YAC_FALSE;
   shmHandle.ptr = NULL;
   b_locked = YAC_FALSE;
#ifdef YAC_WIN32
   hMutex = NULL;
#endif // YAC_WIN32
}

TKS_SharedBuffer::~TKS_SharedBuffer() {
   TKS_SharedBuffer::free();
}

sBool TKS_SharedBuffer::isOwner(void) {
   return shmOwner;
}

void TKS_SharedBuffer::free(void) {
   if(shmHandle.ptr)
   {
      if(b_locked)
         unlock();
      freeSharedMemory();
      shmHandle.ptr = NULL;
      shmOwner      = YAC_FALSE;
      buffer        = NULL;
      io_offset     = 0;
      size          = 0;
      shmKey.empty();
   }
}

sBool TKS_SharedBuffer::allocSharedMemory(sUI _size) {
   TKS_SharedBuffer::free();
   
   genSharedMemoryKey();

#ifdef YAC_WIN32
   // see <https://learn.microsoft.com/en-us/windows/win32/sync/using-named-objects>
   hMutex = ::CreateMutex(NULL,                            // default security descriptor
                          FALSE,                           // mutex not owned
                          TEXT((const char*)shmKey.chars)  // object name
                          );
#endif // YAC_WIN32

   return allocSharedMemoryByKey(&shmKey, _size);
}

void TKS_SharedBuffer::genSharedMemoryKey(void) {
   time_t t; time(&t); 
   sUI rand_ms = (sUI)t;
   shmKey.alloc(18);
   shmKey.printf("%08x%08x", rand_ms, &rand_ms);
}

YAC_String *TKS_SharedBuffer::getSharedMemoryKey (void) {
   return &shmKey;
}

sBool TKS_SharedBuffer::setSize(sUI _size) {
   Dprintf("[---] warning: setSize(size=%u) called on class SharedBuffer.\n", _size);
   return YAC_FALSE;
}

sBool TKS_SharedBuffer::resize(sUI _size) {
   Dprintf("[---] warning: resize(size=%u) called on class SharedBuffer.\n", _size);
   return YAC_FALSE;
}

sBool YAC_VCALL TKS_SharedBuffer::yacArrayAlloc(sUI _sx, sUI _sy, sUI _type, sUI _elementByteSize) {
   Dprintf("[---] warning: yacArrayAlloc(sx=%i sy=%i type=%i elementByteSize=%i) called on SharedBuffer.\n", 
      _sx, _sy, _type, _elementByteSize);
   return YAC_FALSE;
}

sBool YAC_VCALL TKS_SharedBuffer::yacArrayRealloc(sUI _sx, sUI _sy, sUI _type, sUI _elementByteSize) {
   Dprintf("[---] warning: yacArrayRealloc(sx=%i sy=%i type=%i elementByteSize=%i) called on SharedBuffer.\n", 
      _sx, _sy, _type, _elementByteSize);
   return YAC_FALSE;
}
